@@ -2,7 +2,7 @@ import dataclasses
 import enum
 from dataclasses import dataclass
 from enum import Enum, Flag, auto
-from typing import Optional
+from typing import Any, Callable, Optional
 
 import numpy as np
 
@@ -125,6 +125,27 @@ class DerivCheck(Flag):
     CheckAll = CheckFirst | CheckSecond
 
 
+class ScalingType(Enum):
+    """
+    How to scale the problem
+    """
+
+    NoScaling = auto()
+    """
+    No scaling
+    """
+
+    GradJac = auto()
+    """
+    Scale based on gradient and equilibration of constraint Jacobian
+    """
+
+    Custom = auto()
+    """
+    User-provided custom scaling
+    """
+
+
 @dataclass
 class Params:
     """
@@ -158,7 +179,7 @@ class Params:
 
     step_control_type: StepControlType = StepControlType.DistanceRatio
 
-    step_solver: object = None
+    step_solver: Optional[Callable[..., Any]] = None
     step_solver_type: StepSolverType = StepSolverType.Symmetric
     linear_solver_type: LinearSolverType = LinearSolverType.LU
     penalty_update: PenaltyUpdate = PenaltyUpdate.DualNorm
@@ -169,6 +190,9 @@ class Params:
 
     precision: Precision = Precision.Double
 
+    scaling_type: ScalingType = ScalingType.NoScaling
+    scaling: object = None
+
     validate_input: bool = True
 
     iteration_limit: Optional[int] = None
