@@ -1,5 +1,6 @@
 import abc
 import functools
+from typing import Optional, cast
 
 import numpy as np
 import scipy as sp
@@ -43,13 +44,36 @@ class StepSolver(abc.ABC):
         self.n = problem.num_vars
         self.m = problem.num_cons
 
+        self._active_set: Optional[np.ndarray] = None
+        self._jac: Optional[sp.sparse.spmatrix] = None
+        self._hess: Optional[sp.sparse.spmatrix] = None
+
+        self.solver: Optional[LinearSolver] = None
+
+    @property
+    def active_set(self) -> np.ndarray:
+        assert self._active_set is not None
+        return cast(np.ndarray, self._active_set)
+
+    @property
+    def jac(self) -> sp.sparse.spmatrix:
+        assert self._jac is not None
+        return cast(sp.sparse.spmatrix, self._jac)
+
+    @property
+    def hess(self) -> sp.sparse.spmatrix:
+        assert self._hess is not None
+        return cast(sp.sparse.spmatrix, self._hess)
+
     def linear_solver(self, mat: sp.sparse.spmatrix) -> LinearSolver:
         from .linear_solver import linear_solver
 
         solver_type = self.params.linear_solver_type
         return linear_solver(mat, solver_type)
 
-    def estimate_rcond(self, mat: sp.sparse.spmatrix, solver: LinearSolver) -> float:
+    def estimate_rcond(
+        self, mat: sp.sparse.spmatrix, solver: LinearSolver
+    ) -> Optional[float]:
         from .cond_estimate import ConditionEstimator
 
         estimator = ConditionEstimator(mat, solver, self.params)
